@@ -19,6 +19,7 @@
   def (LINE_UPDATE)
   def (ADD_LINES)
   def (OSC_SEQ)
+  def (OSC_SEQ_PERL)
   def (TT_WRITE)
 
   def (REFRESH_BEGIN)
@@ -26,6 +27,7 @@
 
   def (USER_COMMAND)
 
+  def (ROOT_EVENT)
   def (X_EVENT)
   def (FOCUS_IN)
   def (FOCUS_OUT)
@@ -41,6 +43,7 @@
   def (WM_DELETE_WINDOW)
   def (WM_PROTOCOLS)
   def (PROPERTY_NOTIFY)
+  def (RESIZE_ALL_WINDOWS)
 
   def (XIM_PREEDIT_START)
   def (XIM_PREEDIT_DONE)
@@ -49,3 +52,7 @@
 
   def (CUSTOM_REND) // hovering over custom rendition, generate enter/leave maybe?
 
+  def (BELL)
+
+  def (TT_PASTE)
+
